local HC = require 'hardoncollider'
-- has to come before the sti initialization so I can add the collision tiles
collider = HC(100)

class = require '30log'

local sti = require "sti"

local character = require "character"
local ui = require "ui"

local characters = {}

function love.load(arg)
  if arg[#arg] == "-debug" then require("mobdebug").start() end -- debugging in ZeroBraineStudio
  
  windowWidth = love.graphics.getWidth()
  windowHeight = love.graphics.getHeight()

  map = sti.new("assets/map")
  collision = map:getCollisionMap("collision")

  initCharacters()
  ui:showMessage("Sed ut perspiciatis unde omnis iste natus error sit voluptatem accusantium doloremque laudantium, totam rem aperiam, eaque ipsa quae ab illo inventore veritatis et quasi architecto beatae vitae dicta sunt explicabo. Nemo enim ipsam voluptatem quia voluptas sit aspernatur aut odit aut fugit, sed quia consequuntur magni dolores eos qui ratione voluptatem sequi nesciunt. Neque porro quisquam est, qui dolorem ipsum quia dolor sit amet, consectetur, adipisci velit, sed quia non numquam eius modi tempora incidunt ut labore et dolore magnam aliquam quaerat voluptatem. Ut enim ad minima veniam, quis nostrum exercitationem ullam corporis suscipit laboriosam, nisi ut aliquid ex ea commodi consequatur? Quis autem vel eum iure reprehenderit qui in ea voluptate velit esse quam nihil molestiae consequatur, vel illum qui dolorem eum fugiat quo voluptas nulla pariatur?")
end

function love.update(dt)
  if not ui.active then
    for _, char in ipairs(characters) do
      char.controller.animation:update(dt)

      if char.relevantKeys then
        for _, relevantKey in ipairs(char.relevantKeys) do
          if love.keyboard.isDown(relevantKey) then
            char.controller:sendKey(relevantKey)
          end
        end
      end
    end
  else
    for _, relevantKey in ipairs(ui.relevantKeys) do
      if love.keyboard.isDown(relevantKey) then
        ui:sendKey(relevantKey)
      end
    end
  end

  collider:update(dt)
  map:update(dt)
end

function love.keyreleased(key)
  for _, char in ipairs(characters) do
    if char.relevantKeys then
      for _, relevantKey in ipairs(char.relevantKeys) do
        if key == relevantKey then
          char.controller:stopAnimation()
        end
      end
    end
  end
end

function love.draw()
  -- Translation would normally be based on a player's x/y
  local translateX = 0
  local translateY = 0

  -- Draw Range culls unnecessary tiles
  map:setDrawRange(translateX, translateY, windowWidth, windowHeight)

  map:draw()
  ui:draw()

  --debugDrawing()
end

function debugDrawing()
  -- draw collision shapes for debugging
  for shape in pairs(collider:shapesInRange(0, 0, windowWidth, windowHeight)) do
    shape:draw()
  end

  -- Draw Collision Map (useful for debugging)
  map:drawCollisionMap(collision)
end

function initCharacters()
  map:addCustomLayer("character layer", 4)

  local characterLayer = map.layers["character layer"]
  characterLayer.sprites = {}

  for _, char in pairs(map.layers["characters"].objects) do
    local filename = "characters/" .. char.name .. ".lua"
    local charinfo = love.filesystem.load(filename)()
    charinfo.x = char.x - character.width / 4
    charinfo.y = char.y

    charinfo.controller.charinfo = charinfo

    charinfo.image = love.graphics.newImage(charinfo.spritesheet)
    charinfo.controller:init()

    table.insert(characterLayer.sprites, {info = charinfo})
    table.insert(characters, charinfo)
  end

  -- Draw callback for Custom Layer
  function characterLayer:draw()
    table.sort(self.sprites, character.sortNorthSouth)

    for _, sprite in pairs(self.sprites) do
      local x = math.floor(sprite.info.x)
      local y = math.floor(sprite.info.y)

      sprite.info.controller.animation:draw(sprite.info.image, x, y)
    end
  end

  map.layers["characters"].visible = false
end
